@@ -8,8 +8,12 @@ PROPKA representation of bonds.
 import logging
 import math
 import json
-import pkg_resources
+from pathlib import Path
 import propka.calculations
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from propka.molecular_container import MolecularContainer
 
 
 _LOGGER = logging.getLogger(__name__)
@@ -31,6 +35,8 @@ class BondMaker:
     TODO - the documentation for this class needs to be improved.
     """
     def __init__(self):
+        from propka.input import open_file_for_reading
+
         # predefined bonding distances
         self.distances = {'S-S': DISULFIDE_DISTANCE,
                           'F-F': FLUORIDE_DISTANCE}
@@ -47,9 +53,8 @@ class BondMaker:
             + [self.default_dist_squared])
         self.max_sq_distance = max(distances)
         # protein bonding data
-        self.data_file_name = (
-            pkg_resources.resource_filename(__name__, 'protein_bonds.json'))
-        with open(self.data_file_name, 'rt') as json_file:
+        self.data_file_name = Path(__file__).parent / 'protein_bonds.json'
+        with open_file_for_reading(self.data_file_name) as json_file:
             self.protein_bonds = json.load(json_file)
         self.intra_residue_backbone_bonds = {'N': ['CA'], 'CA': ['N', 'C'],
                                              'C': ['CA', 'O'], 'O': ['C']}
@@ -88,6 +93,7 @@ class BondMaker:
         Args:
             protein:  the protein to search for bonds
         """
+        raise NotImplementedError("unused")
         _LOGGER.info('++++ Side chains ++++')
         # side chains
         for chain in protein.chains:
@@ -127,6 +133,7 @@ class BondMaker:
             cys1:  one of the cysteines to check
             cys1:  one of the cysteines to check
         """
+        raise NotImplementedError("unused")
         for atom1 in cys1.atoms:
             if atom1.name == 'SG':
                 for atom2 in cys2.atoms:
@@ -329,7 +336,7 @@ class BondMaker:
                 return True
         return False
 
-    def find_bonds_for_molecules_using_boxes(self, molecules):
+    def find_bonds_for_molecules_using_boxes(self, molecules: "MolecularContainer"):
         """ Finds all bonds for a molecular container.
 
         Args:
