@@ -6,6 +6,15 @@ Container data structure for molecular conformations.
 """
 import logging
 import functools
+from typing import Callable, Dict, Iterable, Iterator, List, NoReturn, Optional, TYPE_CHECKING, Set
+
+from propka.lib import Options
+from propka.version import Version
+
+if TYPE_CHECKING:
+    from propka.atom import Atom
+    from propka.molecular_container import MolecularContainer
+
 import propka.ligand
 from propka.output import make_interaction_map
 from propka.determinant import Determinant
@@ -13,10 +22,13 @@ from propka.coupled_groups import NCCG
 from propka.determinants import set_backbone_determinants, set_ion_determinants
 from propka.determinants import set_determinants
 from propka.group import Group, is_group
+from propka.parameters import Parameters
 
 
 _LOGGER = logging.getLogger(__name__)
 
+CallableGroupToGroups = Callable[[Group], List[Group]]
+
 
 #: A large number that gets multipled with the integer obtained from applying
 #: :func:`ord` to the atom chain ID.  Used in calculating atom keys for
@@ -37,7 +49,10 @@ class ConformationContainer:
        PROPKA inputs is no longer supported.
     """
 
-    def __init__(self, name='', parameters=None, molecular_container=None):
+    def __init__(self,
+                 name: str,
+                 parameters: Parameters,
+                 molecular_container: "MolecularContainer"):
         """Initialize conformation container.
 
         Args:
@@ -48,9 +63,9 @@ class ConformationContainer:
         self.molecular_container = molecular_container
         self.name = name
         self.parameters = parameters
-        self.atoms = []
-        self.groups = []
-        self.chains = []
+        self.atoms: List["Atom"] = []
+        self.groups: List[Group] = []
+        self.chains: List[str] = []
         self.current_iter_item = 0
         self.marvin_pkas_calculated = False
         self.non_covalently_coupled_groups = False
@@ -125,7 +140,8 @@ class ConformationContainer:
                            self.get_titratable_groups()))) > 0:
             self.non_covalently_coupled_groups = True
 
-    def find_bonded_titratable_groups(self, atom, num_bonds, original_atom):
+    def find_bonded_titratable_groups(self, atom: "Atom", num_bonds: int,
+                                      original_atom: "Atom"):
         """Find bonded titrable groups.
 
         Args:
@@ -135,7 +151,8 @@ class ConformationContainer:
         Returns:
             a set of bonded atom groups
         """
-        res = set()
+        assert self.parameters is not None
+        res: Set[Group] = set()
         for bond_atom in atom.bonded_atoms:
             # skip the original atom
             if bond_atom == original_atom:
@@ -151,7 +168,7 @@ class ConformationContainer:
                     bond_atom, num_bonds+1, original_atom)
         return res
 
-    def setup_and_add_group(self, group):
+    def setup_and_add_group(self, group: Optional[Group]):
         """Check if we want to include this group in the calculations.
 
         Args:
@@ -165,7 +182,7 @@ class ConformationContainer:
         self.init_group(group)
         self.groups.append(group)
 
-    def init_group(self, group):
+    def init_group(self, group: Group):
         """Initialize the given Group object.
 
         Args:
@@ -177,15 +194,16 @@ class ConformationContainer:
 
         # If --titrate_only option is set, make non-specified residues
         # un-titratable:
+        assert self.molecular_container.options is not None
         titrate_only = self.molecular_container.options.titrate_only
         if titrate_only is not None:
             atom = group.atom
-            if not (atom.chain_id, atom.res_num, atom.icode) in titrate_only:
+            if (atom.chain_id, atom.res_num, atom.icode) not in titrate_only:
                 group.titratable = False
                 if group.residue_type == 'CYS':
                     group.exclude_cys_from_results = True
 
-    def calculate_pka(self, version, options):
+    def calculate_pka(self, version: Version, options: Options):
         """Calculate pKas for conformation container.
 
         Args:
@@ -261,7 +279,7 @@ class ConformationContainer:
         return penalised_labels
 
     @staticmethod
-    def share_determinants(groups):
+    def share_determinants(groups: Iterable[Group]):
         """Share sidechain, backbone, and Coloumb determinants between groups.
 
         Args:
@@ -271,7 +289,7 @@ class ConformationContainer:
         types = ['sidechain', 'backbone', 'coulomb']
         for type_ in types:
             # find maximum value for each determinant
-            max_dets = {}
+            max_dets: Dict[Group, float] = {}
             for group in groups:
                 for det in group.determinants[type_]:
                     # update max dets
@@ -287,7 +305,11 @@ class ConformationContainer:
                 for group in groups:
                     group.set_determinant(new_determinant, type_)
 
-    def get_coupled_systems(self, groups, get_coupled_groups):
+    def get_coupled_systems(
+        self,
+        groups: Iterable[Group],
+        get_coupled_groups: CallableGroupToGroups,
+    ) -> Iterator[Set[Group]]:
         """A generator that yields covalently coupled systems.
 
         Args:
@@ -299,15 +321,16 @@ class ConformationContainer:
         groups = set(groups)
         while len(groups) > 0:
             # extract a system of coupled groups ...
-            system = set()
+            system: Set[Group] = set()
             self.get_a_coupled_system_of_groups(
                 groups.pop(), system, get_coupled_groups)
             # ... and remove them from the list
             groups -= system
             yield system
 
-    def get_a_coupled_system_of_groups(self, new_group, coupled_groups,
-                                       get_coupled_groups):
+    def get_a_coupled_system_of_groups(self, new_group: Group,
+                                       coupled_groups: Set[Group],
+                                       get_coupled_groups: CallableGroupToGroups):
         """Set up coupled systems of groups.
 
         Args:
@@ -338,7 +361,7 @@ class ConformationContainer:
                                                   reference=reference)
         return ddg
 
-    def calculate_charge(self, parameters, ph=None):
+    def calculate_charge(self, parameters: Parameters, ph: float):
         """Calculate charge for folded and unfolded states.
 
         Args:
@@ -356,7 +379,7 @@ class ConformationContainer:
                                              state='folded')
         return unfolded, folded
 
-    def get_backbone_groups(self):
+    def get_backbone_groups(self) -> List[Group]:
         """Get backbone groups needed for the pKa calculations.
 
         Returns:
@@ -474,7 +497,7 @@ class ConformationContainer:
             group for group in self.groups
             if group.residue_type in self.parameters.ions.keys()]
 
-    def get_group_names(self, group_list):
+    def get_group_names(self, group_list: NoReturn) -> NoReturn:  # FIXME unused?
         """Get names of groups in list.
 
         Args:
@@ -482,9 +505,11 @@ class ConformationContainer:
         Returns:
             list of groups
         """
+        if TYPE_CHECKING:
+            assert False
         return [group for group in self.groups if group.type in group_list]
 
-    def get_ligand_atoms(self):
+    def get_ligand_atoms(self) -> List["Atom"]:
         """Get atoms associated with ligands.
 
         Returns:
@@ -492,7 +517,7 @@ class ConformationContainer:
         """
         return [atom for atom in self.atoms if atom.type == 'hetatm']
 
-    def get_heavy_ligand_atoms(self):
+    def get_heavy_ligand_atoms(self) -> List["Atom"]:
         """Get heavy atoms associated with ligands.
 
         Returns:
@@ -502,7 +527,7 @@ class ConformationContainer:
             atom for atom in self.atoms
             if atom.type == 'hetatm' and atom.element != 'H']
 
-    def get_chain(self, chain):
+    def get_chain(self, chain: str) -> List["Atom"]:
         """Get atoms associated with a specific chain.
 
         Args:
@@ -512,7 +537,7 @@ class ConformationContainer:
         """
         return [atom for atom in self.atoms if atom.chain_id != chain]
 
-    def add_atom(self, atom):
+    def add_atom(self, atom: "Atom"):
         """Add atom to container.
 
         Args:
@@ -553,9 +578,22 @@ class ConformationContainer:
         Args:
             other:  conformation container with atoms to add
         """
+        self.top_up_from_atoms(other.atoms)
+
+    def top_up_from_atoms(self, other_atoms: Iterable["Atom"]):
+        """Adds atoms which are missing from this container.
+
+        Args:
+            other_atoms: Reference atoms
+        """
         my_residue_labels = {a.residue_label for a in self.atoms}
-        for atom in other.atoms:
+        res_names = {(a.chain_id, a.res_num): a.res_name for a in self.atoms}
+        for atom in other_atoms:
             if atom.residue_label not in my_residue_labels:
+                if res_names.setdefault((atom.chain_id, atom.res_num),
+                                        atom.res_name) != atom.res_name:
+                    # don't merge different residue types, e.g. alt-loc mutant
+                    continue
                 self.copy_atom(atom)
 
     def find_group(self, group):
@@ -599,7 +637,7 @@ class ConformationContainer:
             self.atoms[i].numb = i+1
 
     @staticmethod
-    def sort_atoms_key(atom):
+    def sort_atoms_key(atom: "Atom") -> float:
         """Generate key for atom sorting.
 
         Args:
