@@ -7,6 +7,16 @@ Energy calculations.
 """
 import math
 import logging
+from typing import TYPE_CHECKING, Optional, Sequence
+
+from propka.atom import Atom
+from propka.parameters import Parameters
+
+if TYPE_CHECKING:
+    from propka.conformation_container import ConformationContainer
+    from propka.group import Group
+    from propka.version import Version
+
 from propka.calculations import squared_distance, get_smallest_distance
 
 
@@ -27,13 +37,14 @@ COMBINED_NUM_BURIED_MAX = 900
 SEPARATE_NUM_BURIED_MAX = 400
 
 
-def radial_volume_desolvation(parameters, group):
+def radial_volume_desolvation(parameters, group: "Group") -> None:
     """Calculate desolvation terms for group.
 
     Args:
         parameters:  parameters for desolvation calculation
         group:  group of atoms for calculation
     """
+    assert group.atom.conformation_container is not None
     all_atoms = group.atom.conformation_container.get_non_hydrogen_atoms()
     volume = 0.0
     group.num_volume = 0
@@ -66,7 +77,7 @@ def radial_volume_desolvation(parameters, group):
         * volume_after_allowance * scale_factor)
 
 
-def calculate_scale_factor(parameters, weight):
+def calculate_scale_factor(parameters, weight: float) -> float:
     """Calculate desolvation scaling factor.
 
     Args:
@@ -82,7 +93,7 @@ def calculate_scale_factor(parameters, weight):
     return scale_factor
 
 
-def calculate_weight(parameters, num_volume):
+def calculate_weight(parameters: Parameters, num_volume: float) -> float:
     """Calculate the atom-based desolvation weight.
 
     TODO - figure out why a similar function exists in version.py
@@ -102,7 +113,7 @@ def calculate_weight(parameters, num_volume):
     return weight
 
 
-def calculate_pair_weight(parameters, num_volume1, num_volume2):
+def calculate_pair_weight(parameters: Parameters, num_volume1: int, num_volume2: int) -> float:
     """Calculate the atom-pair based desolvation weight.
 
     Args:
@@ -120,7 +131,7 @@ def calculate_pair_weight(parameters, num_volume1, num_volume2):
     return weight
 
 
-def hydrogen_bond_energy(dist, dpka_max, cutoffs, f_angle=1.0):
+def hydrogen_bond_energy(dist, dpka_max: float, cutoffs, f_angle=1.0) -> float:
     """Calculate hydrogen-bond interaction pKa shift.
 
     Args:
@@ -141,7 +152,11 @@ def hydrogen_bond_energy(dist, dpka_max, cutoffs, f_angle=1.0):
     return abs(dpka)
 
 
-def angle_distance_factors(atom1=None, atom2=None, atom3=None, center=None):
+def angle_distance_factors(
+        atom1: Optional[Atom] = None,
+        atom2: Atom = None,  # type: ignore[assignment]
+        atom3: Atom = None,  # type: ignore[assignment]
+        center: Optional[Sequence[float]] = None):
     """Calculate distance and angle factors for three atoms for backbone
     interactions.
 
@@ -175,6 +190,7 @@ def angle_distance_factors(atom1=None, atom2=None, atom3=None, center=None):
     dy_32 = dy_32/dist_23
     dz_32 = dz_32/dist_23
     if atom1 is None:
+        assert center is not None
         dx_21 = center[0] - atom2.x
         dy_21 = center[1] - atom2.y
         dz_21 = center[2] - atom2.z
@@ -190,7 +206,7 @@ def angle_distance_factors(atom1=None, atom2=None, atom3=None, center=None):
     return dist_12, f_angle, dist_23
 
 
-def hydrogen_bond_interaction(group1, group2, version):
+def hydrogen_bond_interaction(group1: "Group", group2: "Group", version: "Version"):
     """Calculate energy for hydrogen bond interactions between two groups.
 
     Args:
@@ -206,7 +222,7 @@ def hydrogen_bond_interaction(group1, group2, version):
     [closest_atom1, dist, closest_atom2] = get_smallest_distance(
         atoms1, atoms2
     )
-    if None in [closest_atom1, closest_atom2]:
+    if closest_atom1 is None or closest_atom2 is None:
         _LOGGER.warning(
             'Side chain interaction failed for {0:s} and {1:s}'.format(
                 group1.label, group2.label))
@@ -290,7 +306,7 @@ def electrostatic_interaction(group1, group2, dist, version):
     return value
 
 
-def check_coulomb_pair(parameters, group1, group2, dist):
+def check_coulomb_pair(parameters: Parameters, group1: "Group", group2: "Group", dist: float) -> bool:
     """Checks if this Coulomb interaction should be done.
 
     NOTE - this is a propka2.0 hack
@@ -319,7 +335,7 @@ def check_coulomb_pair(parameters, group1, group2, dist):
     return do_coulomb
 
 
-def coulomb_energy(dist, weight, parameters):
+def coulomb_energy(dist: float, weight: float, parameters) -> float:
     """Calculates the Coulomb interaction pKa shift based on Coulomb's law.
 
     Args:
@@ -340,7 +356,7 @@ def coulomb_energy(dist, weight, parameters):
     return abs(dpka)
 
 
-def backbone_reorganization(_, conformation):
+def backbone_reorganization(_, conformation: "ConformationContainer") -> None:
     """Perform calculations related to backbone reorganizations.
 
     NOTE - this was described in the code as "adding test stuff"
@@ -440,6 +456,12 @@ def check_coo_arg_exception(group_coo, group_arg, version):
         [closest_coo_atom, dist, closest_arg_atom] = get_smallest_distance(
             atoms_coo, atoms_arg
         )
+        if closest_coo_atom is None:
+            _LOGGER.warning(f"COO interaction atoms missing for {group_coo}")
+            continue
+        if closest_arg_atom is None:
+            _LOGGER.warning(f"ARG interaction atoms missing for {group_arg}")
+            continue
         [dpka_max, cutoff] = version.get_hydrogen_bond_parameters(
             closest_coo_atom, closest_arg_atom
         )
