@@ -9,10 +9,16 @@ protons.
 """
 import logging
 import math
+from typing import Iterable, TYPE_CHECKING
+
 import propka.bonds
 import propka.atom
+from propka.atom import Atom
 from propka.vector_algebra import rotate_vector_around_an_axis, Vector
 
+if TYPE_CHECKING:
+    from propka.molecular_container import MolecularContainer
+
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -28,12 +34,24 @@ class Protonate:
         """
         self.verbose = verbose
         self.valence_electrons = {
-            'H': 1, 'He': 2, 'Li': 1, 'Be': 2, 'B': 3, 'C': 4, 'N': 5,
-            'O': 6, 'F': 7, 'Ne': 8, 'Na': 1, 'Mg': 2, 'Al': 3, 'Si': 4,
-            'P': 5, 'S': 6, 'Cl': 7, 'Ar': 8, 'K': 1, 'Ca': 2, 'Sc': 2,
-            'Ti': 2, 'V': 2, 'Cr': 1, 'Mn': 2, 'Fe': 2, 'Co': 2, 'Ni': 2,
-            'Cu': 1, 'Zn': 2, 'Ga': 3, 'Ge': 4, 'As': 5, 'Se': 6, 'Br': 7,
-            'Kr': 8, 'I': 7}
+            "H": 1, "He": 2, "Li": 1, "Be": 2, "B": 3, "C": 4, "N": 5,
+            "O": 6, "F": 7, "Ne": 8, "Na": 1, "Mg": 2, "Al": 3, "Si": 4,
+            "P": 5, "S": 6, "Cl": 7, "Ar": 8, "K": 1, "Ca": 2, "Sc": 2,
+            "Ti": 2, "V": 2, "Cr": 1, "Mn": 2, "Fe": 2, "Co": 2, "Ni": 2,
+            "Cu": 1, "Zn": 2, "Ga": 3, "Ge": 4, "As": 5, "Se": 6, "Br": 7,
+            "Kr": 8, "Rb": 1, "Sr": 2, "Y": 2, "Zr": 2, "Nb": 1, "Mo": 1,
+            "Tc": 2, "Ru": 1, "Rh": 1, "Pd": 8, "Ag": 1, "Cd": 2, "In": 3,
+            "Sn": 4, "Sb": 5, "Te": 6, "I": 7, "Xe": 8, "Cs": 1, "Ba": 2,
+            "La": 2, "Ce": 2, "Pr": 2, "Nd": 2, "Pm": 2, "Sm": 2, "Eu": 2,
+            "Gd": 2, "Tb": 2, "Dy": 2, "Ho": 2, "Er": 2, "Tm": 2, "Yb": 2,
+            "Lu": 2, "Hf": 2, "Ta": 2, "W": 2, "Re": 2, "Os": 2, "Ir": 2,
+            "Pt": 1, "Au": 1, "Hg": 2, "Tl": 3, "Pb": 4, "Bi": 5, "Po": 6,
+            "At": 7, "Rn": 8, "Fr": 1, "Ra": 2, "Ac": 2, "Th": 2, "Pa": 2,
+            "U": 2, "Np": 2, "Pu": 2, "Am": 2, "Cm": 2, "Bk": 2,  "Cf": 2,
+            "Es": 2, "Fm": 2, "Md": 2, "No": 2, "Lr": 3, "Rf": 2, "Db": 2,
+            "Sg": 2, "Bh": 2, "Hs": 2, "Mt": 2, "Ds": 2, "Rg": 2, "Cn": 2,
+            "Nh": 3, "Fl": 4, "Mc": 5, "Lv": 6, "Ts": 7, "Og": 8, "Uue": 1
+            }
         # TODO - consider putting charges in a configuration file
         self.standard_charges = {
             'ARG-NH1': 1.0, 'ASP-OD2': -1.0, 'GLU-OE2': -1.0, 'HIS-ND1': 1.0,
@@ -46,7 +64,7 @@ class Protonate:
             'Br': 1.41, 'I': 1.61, 'S': 1.35}
         self.protonation_methods = {4: self.tetrahedral, 3: self.trigonal}
 
-    def protonate(self, molecules):
+    def protonate(self, molecules: "MolecularContainer"):
         """Protonate all atoms in the molecular container.
 
         Args:
@@ -63,7 +81,7 @@ class Protonate:
                 self.protonate_atom(atom)
 
     @staticmethod
-    def remove_all_hydrogen_atoms(molecular_container):
+    def remove_all_hydrogen_atoms(molecular_container: "MolecularContainer"):
         """Remove all hydrogen atoms from molecule.
 
         Args:
@@ -74,7 +92,7 @@ class Protonate:
                 molecular_container.conformations[name]
                 .get_non_hydrogen_atoms())
 
-    def set_charge(self, atom):
+    def set_charge(self, atom: Atom):
         """Set charge for atom.
 
         Args:
@@ -97,7 +115,7 @@ class Protonate:
                 atom.sybyl_type = atom.sybyl_type.replace('-', '')
                 atom.charge_set = True
 
-    def protonate_atom(self, atom):
+    def protonate_atom(self, atom: Atom):
         """Protonate an atom.
 
         Args:
@@ -114,7 +132,7 @@ class Protonate:
         atom.is_protonated = True
 
     @staticmethod
-    def set_proton_names(heavy_atoms):
+    def set_proton_names(heavy_atoms: Iterable[Atom]):
         """Set names for protons.
 
         Args:
@@ -127,7 +145,7 @@ class Protonate:
                     bonded.name += str(i)
                     i += 1
 
-    def set_number_of_protons_to_add(self, atom):
+    def set_number_of_protons_to_add(self, atom: Atom):
         """Set the number of protons to add to this atom.
 
         Args:
@@ -137,6 +155,10 @@ class Protonate:
         _LOGGER.debug('Setting number of protons to add for %s', atom)
         atom.number_of_protons_to_add = 8
         _LOGGER.debug("                     8")
+        if atom.element not in self.valence_electrons:
+            _LOGGER.warning(
+                    f'Unknown valence electron for element {atom.element}')
+            self.valence_electrons[atom.element] = 4
         atom.number_of_protons_to_add -= self.valence_electrons[atom.element]
         _LOGGER.debug('Valence electrons: {0:>4d}'.format(
             -self.valence_electrons[atom.element]))
@@ -153,7 +175,7 @@ class Protonate:
         _LOGGER.debug('-'*10)
         _LOGGER.debug(atom.number_of_protons_to_add)
 
-    def set_steric_number_and_lone_pairs(self, atom):
+    def set_steric_number_and_lone_pairs(self, atom: Atom):
         """Set steric number and lone pairs for atom.
 
         Args:
@@ -165,6 +187,10 @@ class Protonate:
         _LOGGER.debug('='*10)
         _LOGGER.debug('Setting steric number and lone pairs for %s', atom)
         atom.steric_number = 0
+        if atom.element not in self.valence_electrons:
+            self.valence_electrons[atom.element] = 4
+            _LOGGER.warning(
+                    f"Not found valence for element {atom.element}, use 4")
         _LOGGER.debug('{0:>65s}: {1:>4d}'.format(
             'Valence electrons', self.valence_electrons[atom.element]))
         atom.steric_number += self.valence_electrons[atom.element]
@@ -187,8 +213,7 @@ class Protonate:
         atom.steric_number += 0
         _LOGGER.debug('{0:>65s}: {1:>4.1f}'.format(
             'Charge(-)', atom.charge))
-        atom.steric_number -= atom.charge
-        atom.steric_number = math.floor(atom.steric_number/2.0)
+        atom.steric_number = math.floor((atom.steric_number - atom.charge) / 2)
         atom.number_of_lone_pairs = (
             atom.steric_number - len(atom.bonded_atoms)
             - atom.number_of_protons_to_add
@@ -200,7 +225,7 @@ class Protonate:
             'Number of lone pairs', atom.number_of_lone_pairs))
         atom.steric_num_lone_pairs_set = True
 
-    def add_protons(self, atom):
+    def add_protons(self, atom: Atom):
         """Add protons to atom.
 
         Args:
@@ -216,7 +241,7 @@ class Protonate:
                 '(steric number: {0:d})'.format(atom.steric_number)
             )
 
-    def trigonal(self, atom):
+    def trigonal(self, atom: Atom):
         """Add hydrogens in trigonal geometry.
 
         Args:
@@ -249,15 +274,15 @@ class Protonate:
                 vec2 = Vector(atom1=atom.bonded_atoms[0],
                               atom2=atom.bonded_atoms[0]
                               .bonded_atoms[other_atom_indices[0]])
-                axis = vec1**vec2
+                axis = vec1.cross(vec2)
                 # this is a trick to make sure that the order of atoms doesn't
                 # influence the final postions of added protons
                 if len(other_atom_indices) > 1:
                     vec3 = Vector(atom1=atom.bonded_atoms[0],
                                   atom2=atom.bonded_atoms[0]
                                   .bonded_atoms[other_atom_indices[1]])
-                    axis2 = vec1**vec3
-                    if axis*axis2 > 0:
+                    axis2 = vec1.cross(vec3)
+                    if axis.dot(axis2) > 0:
                         axis = axis+axis2
                     else:
                         axis = axis-axis2
@@ -276,7 +301,7 @@ class Protonate:
             new_a = self.set_bond_distance(new_a, atom.element)
             self.add_proton(atom, cvec+new_a)
 
-    def tetrahedral(self, atom):
+    def tetrahedral(self, atom: Atom):
         """Protonate atom in tetrahedral geometry.
 
         Args:
@@ -318,13 +343,14 @@ class Protonate:
             self.add_proton(atom, cvec+new_a)
 
     @staticmethod
-    def add_proton(atom, position):
+    def add_proton(atom: Atom, position: Vector):
         """Add a proton to an atom at a specific position.
 
         Args:
             atom:  atom to protonate
             position:  position for proton
         """
+        assert atom.conformation_container is not None
         # Create the new proton
         new_h = propka.atom.Atom()
         new_h.set_property(
@@ -348,7 +374,6 @@ class Protonate:
         new_h.number_of_lone_pairs = 0
         new_h.number_of_protons_to_add = 0
         new_h.num_pi_elec_2_3_bonds = 0
-        new_h.is_protonates = True
         atom.bonded_atoms.append(new_h)
         atom.number_of_protons_to_add -= 1
         atom.conformation_container.add_atom(new_h)
@@ -366,7 +391,7 @@ class Protonate:
                 i += 1
         _LOGGER.debug('added %s %s %s', new_h, 'to', atom)
 
-    def set_bond_distance(self, bvec, element):
+    def set_bond_distance(self, bvec: Vector, element: str) -> Vector:
         """Set bond distance between atom and element.
 
         Args:
